/*
 * kbd_mode: report and set keyboard mode - aeb, 940406
 * 
 * If you make \215A\201 an alias for "kbd_mode -a", and you are
 * in raw mode, then hitting F7 = (two keys) will return you to sanity.
 */
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/kd.h>

#include <lct/local.h>
#include <lct/utils.h>
#include <lct/console.h>

char* progname;

void usage()
{
  fprintf(stderr, "Usage: %s [-a|-u|-k|-s]\n\n", progname);
}

int main(int argc, char *argv[])
{
  int fd, mode;
  
  progname = strip_path(argv[0]);
  
  if (-1 == (fd = get_console_fd(NULL)))
      exit (1);
  
  if (argc == 1) 
    {
      /* report mode */
      if (ioctl(fd, KDGKBMODE, &mode)) 
	{
	  fprintf(stderr, progname);
	  perror(": error reading keyboard mode\n");
	  exit(1);
	}
      switch(mode) 
	{
	case K_RAW:
	  printf("The keyboard is in raw (scancode) mode\n");
	  break;
	case K_MEDIUMRAW:
	  printf("The keyboard is in mediumraw (keycode) mode\n");
	  break;
	case K_XLATE:
	  printf("The keyboard is in the default (ASCII) mode\n");
	  break;
	case K_UNICODE:
	  printf("The keyboard is in Unicode (UTF-8) mode\n");
	  break;
	default:
	  printf("The keyboard is in some unknown mode\n");
	}
      exit(1);
    }
  if (argc != 2)
      badusage("wrong number of args");
  if (!strcmp(argv[1], "-a"))
      mode = K_XLATE;
  else if (!strcmp(argv[1], "-u"))
      mode = K_UNICODE;
  else if (!strcmp(argv[1], "-s"))
      mode = K_RAW;
  else if (!strcmp(argv[1], "-k"))
      mode = K_MEDIUMRAW;
  else
      badusage("unrecognized argument");
  if (ioctl(fd, KDSKBMODE, mode)) 
    {
      fprintf(stderr, progname);
      perror(": error setting keyboard mode\n");
      exit(1);
    }
  exit(0);
}
